// LineWire.cpp

#include "LineWire.h"
// InfoWindow

// MediaRoutingView
//#include "MediaJack.h"
#include "MediaRoutingDefs.h"

// Support
#include "cortex_ui.h"
//#include "MediaString.h"
// TipManager
//#include "TipManager.h"

// Application Kit
#include <Application.h>
// Interface Kit
#include <MenuItem.h>
#include <PopUpMenu.h>
// Media Kit
#include <MediaDefs.h>

__USE_CORTEX_NAMESPACE

#include <Debug.h>
#define D_METHOD(x) //PRINT (x)
#define D_DRAW(x) //PRINT (x)
#define D_MOUSE(x) //PRINT (x)

// -------------------------------------------------------- //
// constants
// -------------------------------------------------------- //

const float LineWire::M_WIRE_OFFSET		= 0.0;

// -------------------------------------------------------- //
// *** ctor/dtor
// -------------------------------------------------------- //

LineWire::LineWire(
	
	DiagramEndPoint *outputJack,
	DiagramEndPoint *inputJack)
	: DiagramWire(outputJack, inputJack)
{
	D_METHOD(("LineWire::LineWire()\n"));
}

LineWire::LineWire(
	DiagramEndPoint *jack,
	bool isStartPoint)
	: DiagramWire(jack, isStartPoint)
{
	D_METHOD(("LineWire::LineWire(temp)\n"));
}

LineWire::~LineWire()
{
	D_METHOD(("LineWire::~LineWire()\n"));
}

// -------------------------------------------------------- //
// *** derived from DiagramWire (public)
// -------------------------------------------------------- //

void LineWire::attachedToDiagram()
{
	D_METHOD(("LineWire::detachedFromDiagram()\n"));

	endPointMoved(startPoint());
	endPointMoved(endPoint());
}

void LineWire::detachedFromDiagram()
{
	D_METHOD(("LineWire::detachedFromDiagram()\n"));

}

BRect LineWire::frame() const
{
	D_DRAW(("LineWire::frame()\n"));
	return m_frame;
}

void LineWire::drawWire()
{
	D_DRAW(("LineWire::drawWire()\n"));

	rgb_color border = isSelected() ? M_BLUE_COLOR : M_DARK_GRAY_COLOR;
	rgb_color fill = isSelected() ? M_LIGHT_BLUE_COLOR : M_LIGHT_GRAY_COLOR;
	view()->SetPenSize(3.0);
	view()->BeginLineArray(3);
		view()->AddLine(m_startPoint, m_startOffset, border);
		view()->AddLine(m_startOffset, m_endOffset, border);
		view()->AddLine(m_endOffset, m_endPoint, border);
	view()->EndLineArray();
	view()->SetPenSize(1.0);
	view()->BeginLineArray(3);
		view()->AddLine(m_startPoint, m_startOffset, fill);
		view()->AddLine(m_startOffset, m_endOffset, fill);
		view()->AddLine(m_endOffset, m_endPoint, fill);
	view()->EndLineArray();
}


void LineWire::endPointMoved(
	DiagramEndPoint *which)
{
	if (which == startPoint())
	{
		m_startPoint = startConnectionPoint();
		m_startOffset = m_startPoint + BPoint(M_WIRE_OFFSET, 0.0);
		
		m_frame.left = m_startPoint.x < m_endOffset.x ? m_startPoint.x - 2.0: m_endOffset.x - 2.0;
		m_frame.top = m_startPoint.y < m_endOffset.y ? m_startPoint.y - 2.0 : m_endOffset.y - 2.0;
		m_frame.right = m_startOffset.x > m_endPoint.x ? m_startOffset.x + 2.0 : m_endPoint.x + 2.0;
		m_frame.bottom = m_startOffset.y > m_endPoint.y ? m_startOffset.y + 2.0 : m_endPoint.y + 2.0;
	}
	else if (which == endPoint())
	{
		m_endPoint = endConnectionPoint();
		m_endOffset = m_endPoint - BPoint(M_WIRE_OFFSET, 0.0);
		
		
		m_frame.left = m_startPoint.x < m_endOffset.x ? m_startPoint.x - 2.0: m_endOffset.x - 2.0;
		m_frame.top = m_startPoint.y < m_endOffset.y ? m_startPoint.y - 2.0 : m_endOffset.y - 2.0;
		m_frame.right = m_startOffset.x > m_endPoint.x ? m_startOffset.x + 2.0 : m_endPoint.x + 2.0;
		m_frame.bottom = m_startOffset.y > m_endPoint.y ? m_startOffset.y + 2.0 : m_endPoint.y + 2.0;
	}
}



// END -- LineWire.cpp --
