// MediaWire.cpp
#include "MediaWire.h"
#include "MediaRoutingDefs.h"

#include "WireManager.h"

// Support
#include "cortex_ui.h"

// Application Kit
#include <Application.h>
// Interface Kit
#include <MenuItem.h>
#include <PopUpMenu.h>
// Media Kit
#include <MediaDefs.h>

__USE_CORTEX_NAMESPACE

#include <Debug.h>
#define D_METHOD(x) //PRINT (x)
#define D_DRAW(x) //PRINT (x)
#define D_MOUSE(x) //PRINT (x)

#include "StructBox.h"
#include "EndRelation.h"

extern WireManager*	wire_manager;

const pattern M_DASHED = {0xc7,0x8f,0x1f,0x3e,0x7c,0xf8,0xf1,0xe3};

// -------------------------------------------------------- //
// constants
// -------------------------------------------------------- //

const float MediaWire::M_WIRE_OFFSET		= 0.0;

// -------------------------------------------------------- //
// *** ctor/dtor
// -------------------------------------------------------- //

MediaWire::MediaWire(
	
	DiagramEndPoint *outputJack,
	DiagramEndPoint *inputJack,
	EndRelation *r_from,
	EndRelation *r_to,
	int line=0)
	: DiagramWire(outputJack, inputJack),start_box(r_from),end_box(r_to),m_linestyle(line)
{
	D_METHOD(("MediaWire::MediaWire()\n"));
}

MediaWire::MediaWire(
	DiagramEndPoint *jack,
	bool isStartPoint)
	: DiagramWire(jack, isStartPoint)
{
	D_METHOD(("MediaWire::MediaWire(temp)\n"));
}

MediaWire::~MediaWire()
{
	D_METHOD(("MediaWire::~MediaWire()\n"));
}

// -------------------------------------------------------- //
// *** derived from DiagramWire (public)
// -------------------------------------------------------- //

void MediaWire::attachedToDiagram()
{
	D_METHOD(("MediaWire::detachedFromDiagram()\n"));
//	start_box=new EndRelation(BRect(0,0,20,20));
//	end_box=new EndRelation(BRect(0,0,20,20));
	
	endPointMoved(startPoint());
	endPointMoved(endPoint());
}

void MediaWire::detachedFromDiagram()
{
	D_METHOD(("MediaWire::detachedFromDiagram()\n"));

}

BRect MediaWire::frame() const
{
	D_DRAW(("MediaWire::frame()\n"));
	return m_frame | start_box->frame() | end_box->frame();
}

float MediaWire::howCloseTo(
	BPoint point) const
{
	D_MOUSE(("MediaWire::howCloseTo()\n"));
	if (frame().Contains(point))
	{
		BPoint sp = m_startPoint;
		BPoint ep = m_endPoint;
		BPoint so = m_startOffset;
		BPoint eo = m_endOffset;
		BRect wireFrame, startFrame, endFrame;
		wireFrame.left = so.x < eo.x ? so.x : eo.x;
		wireFrame.top = so.y < eo.y ? so.y : eo.y;
		wireFrame.right = so.x > eo.x ? so.x : eo.x;
		wireFrame.bottom = so.y > eo.y ? so.y : eo.y;
		startFrame.Set(sp.x, sp.y, so.x, so.y);
		endFrame.Set(ep.x, ep.y, eo.x, eo.y);
		wireFrame.InsetBy(-1.0, -1.0);
		startFrame.InsetBy(-1.0, -1.0);
		endFrame.InsetBy(-1.0, -1.0);
		if ((wireFrame.Width() <= 5.0) || (wireFrame.Height() <= 5.0) || startFrame.Contains(point) || endFrame.Contains(point))
		{
			return 1.0;
		}
		else
		{
			float length, result;
			length = sqrt(pow(eo.x - so.x, 2) + pow(eo.y - so.y, 2));
 			result = ((so.y - point.y) * (eo.x - so.x)) - ((so.x - point.x) * (eo.y - so.y));
			result = 3.0 - fabs(result / length);
			return result;
		}
	}
	return 0.0;
}

void MediaWire::drawWire()
{
	D_DRAW(("MediaWire::drawWire()\n"));

	rgb_color border = isSelected() ? M_RED_COLOR : M_BLACK_COLOR;
	view()->SetHighColor(border);
	
	if(m_linestyle==NORMAL_LINE)
	view()->StrokeLine(m_startOffset, m_endOffset);
	 else
	view()->StrokeLine(m_startOffset, m_endOffset,M_DASHED);
	
	start_box->draw(view());
	end_box->draw(view());
}

void MediaWire::mouseDown(
	BPoint point,
	uint32 buttons,
	uint32 clicks)
{
	D_MOUSE(("MediaWire::mouseDown()\n"));
	_inherited::mouseDown(point, buttons, clicks);

	switch (buttons)
	{
		case B_SECONDARY_MOUSE_BUTTON:
		{
			if (clicks == 1)
			{
				showContextMenu(point);
			}
		}
	}
}

void MediaWire::mouseOver(
	BPoint point,
	uint32 transit)
{
	D_MOUSE(("MediaWire::mouseOver()\n"));

	if (isDragging())
	{
		return;
	}
	switch (transit)
	{
		case B_ENTERED_VIEW:
		{
			/*TipManager *tips = TipManager::Instance();
			BString tipText = MediaString::getStringFor(connection.format(), false);
			tips->showTip(tipText.String(), view()->ConvertToScreen(frame()), 
						  TipManager::LEFT_OFFSET_FROM_POINTER, BPoint(12.0, 8.0));*/
			be_app->SetCursor(M_CABLE_CURSOR);
			break;
		}
		case B_EXITED_VIEW:
		{
			be_app->SetCursor(B_HAND_CURSOR);
			/*TipManager *tips = TipManager::Instance();
			tips->hideTip(view()->ConvertToScreen(frame()));*/
			break;
		}
	}
}

void MediaWire::selected()
{
	D_METHOD(("MediaWire::selected()\n"));
	if (startPoint())
	{
		DiagramEndPoint *outputJack = dynamic_cast<DiagramEndPoint *>(startPoint());
		outputJack->select();
	}
	if (endPoint())
	{
		DiagramEndPoint *inputJack = dynamic_cast<DiagramEndPoint *>(endPoint());
		inputJack->select();
	}
}

void MediaWire::deselected()
{
	D_METHOD(("MediaWire::deselected()\n"));
	if (startPoint())
	{
		DiagramEndPoint *outputJack = dynamic_cast<DiagramEndPoint *>(startPoint());
		outputJack->deselect();
	}
	if (endPoint())
	{
		DiagramEndPoint *inputJack = dynamic_cast<DiagramEndPoint *>(endPoint());
		inputJack->deselect();
	}
}
void MediaWire::_noName(EndRelation* simb,DiagramEndPoint* which,BPoint p_start,BPoint p_end,BPoint *offset)
{	
		// EndWire of startingPoint
		
		
			
			BRect intersex=(((StructBox*)(which->group()))->frame());
			float m_sex=(((StructBox*)(which->group()))->getM());
			// calcolo della retta m_f
			
			float m_f=(p_end.y-p_start.y)/(p_end.x-p_start.x);
			
			if(p_end.x-p_start.x==0) m_f=(p_end.y-p_start.y)/0.0001; //fixed the NaN bug ?
			
			float c_f=p_end.y-m_f*p_end.x;
			
			//float offset->y,offset->x;
									
			if(m_f>m_sex || m_f<-1*m_sex) //lato oriz (top  or bottom)
			{
				
				
				if(p_start.y<p_end.y)
					
					{					
						offset->y=intersex.bottom;
						offset->x=(offset->y-c_f)/m_f;
						simb->offsetDir(offset->x,offset->y,W_BOTTOM,offset);
						
					}
				else 
					{
						offset->y=intersex.top;
						offset->x=(offset->y-c_f)/m_f;
						simb->offsetDir(offset->x,offset->y,W_TOP,offset);
						
					}
				
			}
			else
			{
			
				if(p_start.x<p_end.x)
				{	
					
					offset->x=(intersex.right);
					offset->y=offset->x*m_f + c_f;
					simb->offsetDir(offset->x,offset->y,W_RIGHT,offset);
				}
				else
				{	
					
					offset->x=(intersex.left);
					offset->y=offset->x*m_f + c_f;
					simb->offsetDir(offset->x,offset->y,W_LEFT,offset);
			
				}
										
			}
		

}
void MediaWire::endPointMoved(
	DiagramEndPoint *which)
{

	m_startPoint = startConnectionPoint();
	m_endPoint = endConnectionPoint();
	
	if(startPoint()) _noName(start_box,startPoint(),m_startPoint,m_endPoint,&m_startOffset);	
	
	if(endPoint())   _noName(end_box,endPoint(),m_endPoint,m_startPoint,&m_endOffset);
	
		
	if (which == startPoint())
	{
		
		//m_startOffset = m_startPoint + BPoint(M_WIRE_OFFSET, 0.0);
		
		
		m_frame.left = m_startPoint.x < m_endOffset.x ? m_startPoint.x - 2.0: m_endOffset.x - 2.0;
		m_frame.top = m_startPoint.y < m_endOffset.y ? m_startPoint.y - 2.0 : m_endOffset.y - 2.0;
		m_frame.right = m_startOffset.x > m_endPoint.x ? m_startOffset.x + 2.0 : m_endPoint.x + 2.0;
		m_frame.bottom = m_startOffset.y > m_endPoint.y ? m_startOffset.y + 2.0 : m_endPoint.y + 2.0;
	}
	else if (which == endPoint())
	{
		
		//m_endPoint = endConnectionPoint();
		//m_endOffset = m_endPoint - BPoint(M_WIRE_OFFSET, 0.0);
		
		m_frame.left = m_startPoint.x < m_endOffset.x ? m_startPoint.x - 2.0: m_endOffset.x - 2.0;
		m_frame.top = m_startPoint.y < m_endOffset.y ? m_startPoint.y - 2.0 : m_endOffset.y - 2.0;
		m_frame.right = m_startOffset.x > m_endPoint.x ? m_startOffset.x + 2.0 : m_endPoint.x + 2.0;
		m_frame.bottom = m_startOffset.y > m_endPoint.y ? m_startOffset.y + 2.0 : m_endPoint.y + 2.0;
	}
	
			
}

// -------------------------------------------------------- //
// *** internal operations (protected)
// -------------------------------------------------------- //

void MediaWire::showContextMenu(
	BPoint point)
{
	D_METHOD(("MediaWire::showContextMenu()\n"));

	BPopUpMenu *menu = new BPopUpMenu("PopUp", false, false, B_ITEMS_IN_COLUMN);
	menu->SetFont(be_plain_font);

	BMenuItem *item;
	BMessage *message;
	
	
	message = new BMessage('wpan');
	message->AddPointer("fromWire",(void*)this);
	
	//menu->AddItem(item = new BMenuItem("Properties", message));
	BString str;
	
	//start
	BMenu *tmenu=wire_manager->CreateMenu(this,START_ENDPOINT);
	tmenu->ItemAt(start_box->getID()+1)->SetMarked(true);
	str  = "start  [";
	str += ((StructBox*)startPoint()->group())->getStruct()->getStructName();
	str += "]";
	tmenu->SetName(str.String());
	menu->AddItem(item = new BMenuItem(tmenu));
	tmenu->SetTargetForItems(view());
	//end
	tmenu=wire_manager->CreateMenu(this,END_ENDPOINT);
	tmenu->ItemAt(end_box->getID()+1)->SetMarked(true);
	str  = "end    [";
	str += ((StructBox*)endPoint()->group())->getStruct()->getStructName();
	str += "]";
	tmenu->SetName(str.String());
	menu->AddItem(item = new BMenuItem(tmenu));
	tmenu->SetTargetForItems(view());
	//line
	menu->AddSeparatorItem();
	tmenu=wire_manager->CreateLineMenu(this);
	tmenu->ItemAt(getLineStyle())->SetMarked(true);
	tmenu->SetName("line style");
	menu->AddItem(item = new BMenuItem(tmenu));
	tmenu->SetTargetForItems(view());
	//delete
	menu->AddSeparatorItem();
	BMessage *message2 = new BMessage('wdel');
	message2->AddPointer("fromWire",(void*)this);
	menu->AddItem(item = new BMenuItem("Delete",message2));	
	
	menu->SetTargetForItems(view());
	view()->ConvertToScreen(&point);
	point -= BPoint(1.0, 1.0);
	message->AddPoint("where",point);
	menu->Go(point, true, true, true);
}

// END -- MediaWire.cpp --
