#include "DiagramView.h"
#include "MediaWire.h"
#include "LineWire.h"
#include "DiagramBox.h"
#include "StructTriple.h"
#include "WireManager.h"
#include "InfoStructWindow.h"
#include "UITools.h"

#include <Window.h>

extern WireManager	*wire_manager;
extern UITools		*ui_tools;
class MyBox : public DiagramView
{
	public:
	MyBox(BRect r):DiagramView(r,"null",false,B_FOLLOW_ALL_SIDES,B_WILL_DRAW)
	{
		rgb_color blue;
		blue.red=240;
		blue.green=240;
		blue.blue=240;
		SetLowColor(blue);
		setBackgroundColor(blue);
	}

		// must be implemented to return an instance of the DiagramWire
	// derived class
	virtual DiagramWire *createWire(
							DiagramEndPoint *from,
							DiagramEndPoint *to) { return new LineWire(from,to); };

	// must be implemented to return an instance of the DiagramWire
	// derived class; this version is for temporary wires used in
	// drag & drop connecting
	virtual DiagramWire *createWire(
							DiagramEndPoint *from) { return new LineWire(from,true); };
							
	
	virtual void AttachedToWindow(){
	
		DiagramView::AttachedToWindow();
		ui_tools->ApplyRegularFont(this);
	
	}

	virtual void MessageReceived(
	BMessage *message)
	{
		switch (message->what)
		{
			
			case 'refr':
			{
				StructBox *box;
				if(message->FindPointer("fromBox",(void**)&box)==B_OK)
				{
					BRect r(0,0,0,0);
					box->StructChanged(&r);
					if(r.Width()!=0 || r.Height()!=0)
					{
						updateDataRect();
						Invalidate(r);
					}
										
				}	
			}
			break;
			case 'wpan':
			{
				MediaWire *wire;
				if(message->FindPointer("fromWire",(void**)&wire)==B_OK)
				{
					BView*	w=wire_manager->getPanel(wire);
					BPoint where=message->FindPoint("where");
					BMessage *newmes= new BMessage('wref');
					newmes->AddPointer("fromBox",(void*)wire);
					if(w){ new InfoStructWindow(w,where,this,newmes); }
					
				}	
			}
			break;
			case 'wdel':
			{
				MediaWire *wire;
				if(message->FindPointer("fromWire",(void**)&wire)==B_OK)
				{
						wire->startPoint()->disconnect(wire);		
						wire->endPoint()->disconnect(wire);
						removeItem(wire);
						Invalidate();
			 	}	
			}
			break;
			case 'pane':
			{
				StructBox *box;
				if(message->FindPointer("fromBox",(void**)&box)==B_OK)
				{
					StructTriple *triple=box->m_triple;
					BView*	w=triple->getPanel(box->getStruct());
					BPoint where=message->FindPoint("where");
					BMessage *newmes= new BMessage('refr');
					newmes->AddPointer("fromBox",(void*)box);
					if(w){ new InfoStructWindow(w,where,this,newmes); }
					
				}	
			}
			break;
			case 'dele':
			{
				StructBox *box;
				if(message->FindPointer("fromBox",(void**)&box)==B_OK)
				{
					for(int32 i=box->getCenterEndPoint()->CountConnectedWires()-1;i>=0;i--)
					{
						MediaWire *mw=(MediaWire*)box->getCenterEndPoint()->wire(i);
						mw->startPoint()->disconnect(mw);		
						mw->endPoint()->disconnect(mw);
						removeItem(mw);
					}
					
					delete box->getStruct();
					removeItem(box);
					Invalidate();
			 }	
			}
			break;
			case 'ziza':
				{
				
				//message->PrintToStream();
				
				DiagramItem *box=NULL;
				message->FindPointer("item",(void**)&box);
				if(!box) printf("NULL BOX?\n");
				BPoint to=message->FindPoint("_drop_point_");
				BPoint from=message->FindPoint("from");
				
				DiagramBox *dbox=((DiagramBox*)(box->group()));
				BRegion m_update;
				
				m_update.Include(dbox->frame());
				
				//BRect old=dbox->frame();
				
				to -= from;
				
				if(to.x<-1*dbox->frame().Width()+50) to.x=0;
				if(to.y<-1*dbox->frame().Height()+50) to.y=0;
				
				dbox->resizeBy(to.x,to.y);
				
				//printf("resizeBy %f %f\n",to.x,to.y);
				
				box->moveBy(to.x,to.y);
				dbox->moveBy(0,0,&m_update);
				
				updateDataRect();
				Invalidate(m_update.Frame());		
				}
				break;
			case 'menu':
				{
					MediaWire* wire;
					int32 index=message->FindInt32("index");
					 int	  endpoint=message->FindInt16("endpoint");
					if(message->FindPointer("wire",(void**)&wire)==B_OK)
					{
						
						wire_manager->SetEndPoint(wire,endpoint,index);
						updateDataRect();
						Invalidate(wire->frame());		
						
					} 
				}
			break;
			case 'lmen':
			{
					MediaWire* wire;
					int32 index=message->FindInt32("index");
					if(message->FindPointer("wire",(void**)&wire)==B_OK)
					{
						wire->setLineStyle(index);
						updateDataRect();
						Invalidate(wire->frame());
					}
			}
			break;
			default:
				DiagramView::MessageReceived(message);
			break;
		}
	}
	
	virtual void		connectionEstablished(
							DiagramEndPoint *from,
							DiagramEndPoint *to)
						{ printf("LINKED\n");
						
	if (from->wire(0)==to->wire(0))
	{
		removeItem(from->wire(0));
		Invalidate(from->wire(0)->frame());
		DiagramEndPoint *endPoint = from->wire(0)->startPoint();
		if (!endPoint)
		{
			endPoint = from->wire(0)->endPoint();
		}
		if (endPoint)
		{
			Invalidate(from->wire(0)->frame());
		}
		from->disconnect();
		to->disconnect();
		
		delete from->wire(0);
		
		StructBox *from_item=(StructBox*)from->group();
		StructBox *to_item=(StructBox*)to->group();
		
		// manca controllo se c'e' gia.. ma serve?
		
		MediaWire *wire=wire_manager->createWire(from_item->getCenterEndPoint(),to_item->getCenterEndPoint()); //,NO_END,NAVIGABILITY_END);
		addItem(wire);
		Invalidate(wire->frame());
		//m_draggedWire = 0;
	}
}
	
 
						
};