#include "WireManager.h"
#include "EndRelation.h"
#include "MediaWire.h"
#include "DiagramEndPoint.h"

#include <TranslationUtils.h>
#include <Box.h>
#include <MenuBar.h>
#include <MenuField.h>
#include "BitmapMenuItem.h"
#include "WireMenuItem.h"
#include "UITools.h"

extern UITools*	ui_tools;

WireManager::WireManager()
{
	end_bitmap=new BList(5);
	end_bitmap->AddItem((void*)ui_tools->LoadBitmap("ends/aggreg"));     //0
	end_bitmap->AddItem((void*)ui_tools->LoadBitmap("ends/comp"));    	//1
	end_bitmap->AddItem((void*)ui_tools->LoadBitmap("ends/navig"));  	//2
	end_bitmap->AddItem((void*)ui_tools->LoadBitmap("ends/subclass"));	//3
	//lines
	end_bitmap->AddItem((void*)ui_tools->LoadBitmap("lines"));	//4
	
	def_from=def_to=NO_END;
	def_line=NORMAL_LINE; //MediWare.h
	
	def_list=new BList(5);
	add_def(NO_END,NO_END,NORMAL_LINE);
	add_def(NO_END,NO_END,DASHED_LINE);
	add_def(NO_END,NAVIGABILITY_END,NORMAL_LINE);
	add_def(AGGREGATION_END,NO_END,NORMAL_LINE);
	add_def(NO_END,SUBLASS_END,NORMAL_LINE); 
		
}
void				
WireManager::add_def(int  s,int e,int l)
{
	def_wire *dw=new def_wire();
	dw->start=s;
	dw->end=e;
	dw->line=l;
	def_list->AddItem((void*)dw);	
}

int
WireManager::get_def(int pos,int f)
{
	def_wire *dw=(def_wire*)def_list->ItemAt(pos);
	if(!dw) return -1;
	if(f==0) return dw->start;
	if(f==1) return dw->end;
	if(f==2) return dw->line;
	return -1;
}

MediaWire*
WireManager::createWire(DiagramEndPoint* d_from,DiagramEndPoint* d_to)
{
	return createWire(d_from,d_to,def_from,def_to);
}

void
WireManager::SetEndPoint(MediaWire* wire,int32 endpoint,int32 relationid)
{
	EndRelation	*old=wire->getRelation(endpoint);
	BBitmap *bit;
	
	relationid--;
	if(relationid!=NO_END)
		bit=(BBitmap*)end_bitmap->ItemAt(relationid);
		
	wire->setRelation(new EndRelation(BRect(0,0,20,20),relationid,bit),endpoint);
	wire->attachedToDiagram();
	
	delete old;
}

MediaWire*
WireManager::createWire(DiagramEndPoint* d_from,DiagramEndPoint* d_to,int32 from,int32 to)
{	
	BBitmap *b_from=NULL;
	BBitmap *b_to=NULL;
	
	if(from!=NO_END)
		b_from=(BBitmap*)end_bitmap->ItemAt(from);
	if(to!=NO_END)
		b_to=(BBitmap*)end_bitmap->ItemAt(to);
		
	EndRelation *r_from=new EndRelation(BRect(0,0,20,20),from,b_from);
	EndRelation *r_to=new EndRelation(BRect(0,0,20,20),to,b_to);
	
	return new MediaWire(d_from,d_to,r_from,r_to,def_line);		
}
BView*
WireManager::getPanel(MediaWire * mw)
{
	BBox *box=new BBox(BRect(0,0,200,200));
	BMenu	*menu=new BMenu("start",20,104);
	
	BMessage *mes=new BMessage('menu');
	
	BitmapMenuItem* mi;
	if(mw->getLineStyle()==NORMAL_LINE)
		mi=new BitmapMenuItem(NULL,(BBitmap*)end_bitmap->ItemAt(4),BRect(0,0,20,20),mes);
    else
    	mi=new BitmapMenuItem(NULL,(BBitmap*)end_bitmap->ItemAt(4),BRect(21,0,41,20),mes);
    
    menu->AddItem(mi,BRect(0,0,20,20));
	for(int i=1;i<5;i++)
	{
	 mi=new BitmapMenuItem(NULL,(BBitmap*)end_bitmap->ItemAt(i-1),BRect(42,0,62,20),mes);
     menu->AddItem(mi,BRect(0,20*i+i,20,20*i+i+20));
	}
	menu->SetRadioMode(true);
	BMenuField *field;
	box->AddChild(field=new BMenuField(BRect(20,20,40,40),NULL,"",menu));
	//box->AddChild(field);
	
	return box; 
};

BMenu*
WireManager::CreateMenu(MediaWire * mw,int32 point)
{
	
	BMenu	*menu;
	menu=new BMenu("_noname_",20,104);
	
		
	BMessage *mes=new BMessage('menu');
	mes->AddInt16("endpoint",point); // START or END
	mes->AddPointer("wire",(void*)mw);
		
	BitmapMenuItem* mi;
	if(mw->getLineStyle()==NORMAL_LINE)
		mi=new BitmapMenuItem(NULL,(BBitmap*)end_bitmap->ItemAt(4),BRect(0,0,20,20),mes);
    else
    	mi=new BitmapMenuItem(NULL,(BBitmap*)end_bitmap->ItemAt(4),BRect(21,0,41,20),mes);
    
    menu->AddItem(mi,BRect(0,0,20,20));
	for(int i=1;i<5;i++)
	{
	 mi=new BitmapMenuItem(NULL,(BBitmap*)end_bitmap->ItemAt(i-1),BRect(42,0,62,20),mes);
     menu->AddItem(mi,BRect(0,20*i+i,20,20*i+i+20));
	}
	menu->SetRadioMode(true);
	
	return menu; 
};
BMenu*
WireManager::CreateLineMenu(MediaWire * mw)
{
	
	BMenu	*menu;
	menu=new BMenu("_noname_",20,44);
	
		
	BMessage *mes=new BMessage('lmen');
	mes->AddPointer("wire",(void*)mw);
		
	BitmapMenuItem* mi;
	mi=new BitmapMenuItem(NULL,(BBitmap*)end_bitmap->ItemAt(4),BRect(0,0,20,20),mes);
    menu->AddItem(mi,BRect(0,0,20,20));
    mi=new BitmapMenuItem(NULL,(BBitmap*)end_bitmap->ItemAt(4),BRect(21,0,41,20),mes);
    menu->AddItem(mi,BRect(0,21,20,41));
	menu->SetRadioMode(true);
	return menu; 
};

BMenu*
WireManager::CreateDefaultWiresMenu()
{
	
	BMenu	*menu;
	menu=new BMenu("_noname_",84,def_list->CountItems()*20+def_list->CountItems());
	
		
	BMessage *mes=new BMessage('defw');
	
		
	WireMenuItem* mi;
	
	for(int i=0;i<def_list->CountItems();i++)
	{	
	 mi=new WireMenuItem(getBitmap(get_def(i,0)),getBitmap(get_def(i,1)),getBitmap(LINE),get_def(i,2),mes);
     menu->AddItem(mi,BRect(0,20*i+i,84,20*i+i+20));
    }
  
	menu->SetRadioMode(true);
	return menu; 
};
void
WireManager::SetDefaultWire(int index)
{
	def_wire* dw=(def_wire*)def_list->ItemAt(index);
	if(!dw) return;
	
	def_from=dw->start;
	def_to=dw->end; 
	def_line=dw->line;
 				
}

	